<?php

namespace qfq;

require_once(__DIR__ . '/../database/AbstractDatabaseTest.php');

const TOTAL_COUNT_PERSON_GENERIC_SQL = 2;

/**
 * Created by PhpStorm.
 * User: crose
 * Date: 2/2/16
 * Time: 10:07 PM
 */
class ReportTest extends AbstractDatabaseTest {

    /**
     * @var Evaluate
     */
    private $eval = null;

    /**
     * @var Report
     */
    private $report = null;

    /**
     *
     */
    public function testReportSingleQuery() {

        //empty
        $result = $this->report->process("");
        $this->assertEquals('', $result);

        // comment
        $result = $this->report->process('# 10.sql = SELECT "Hello World"');
        $this->assertEquals('', $result);

        // empty (missing '=')
        $result = $this->report->process('10.sql SELECT "Hello World"');
        $this->assertEquals('', $result);

        // sql
        $result = $this->report->process('10.sql = SELECT "Hello World"');
        $this->assertEquals('Hello World', $result);

        // sql, 2 rows
        $result = $this->report->process('10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2');
        $this->assertEquals('DoeJohnSmithJane', $result);

        // sql, 2 rows, head
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>");
        $this->assertEquals('<table>DoeJohnSmithJane', $result);

        // sql, 2 rows, head, tail
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>");
        $this->assertEquals('<table>DoeJohnSmithJane</table>', $result);

        // sql, 2 rows, head, tail, rbeg
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>");
        $this->assertEquals('<table><tr>DoeJohn<tr>SmithJane</table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = </tr>");
        $this->assertEquals('<table><tr>DoeJohn</tr><tr>SmithJane</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = </tr>\n10.fbeg = <td>");
        $this->assertEquals('<table><tr><td>Doe<td>John</tr><tr><td>Smith<td>Jane</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = </tr>\n10.fbeg = <td>\n10.fend = </td>");
        $this->assertEquals('<table><tr><td>Doe</td><td>John</td></tr><tr><td>Smith</td><td>Jane</td></tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend, renr
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>");
        $this->assertEquals('<table><tr><td>Doe</td><td>John</td>++</tr><tr><td>Smith</td><td>Jane</td>++</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend, renr, rsep
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @");
        $this->assertEquals('<table><tr><td>Doe</td><td>John</td>++</tr>@<tr><td>Smith</td><td>Jane</td>++</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend, renr, rsep, fsep
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @\n10.fsep = $$");
        $this->assertEquals('<table><tr><td>Doe</td>$$<td>John</td>++</tr>@<tr><td>Smith</td>$$<td>Jane</td>++</tr></table>', $result);

        // no 'sql'
        $result = $this->report->process("10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @\n10.fsep = $$");
        $this->assertEquals('', $result);

        // head, tail, rbeg, rend, fbeg, fend, renr, rsep, fsep, sql 2 rows
        $result = $this->report->process("10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @\n10.fsep = $$\n10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2");
        $this->assertEquals('<table><tr><td>Doe</td>$$<td>John</td>++</tr>@<tr><td>Smith</td>$$<td>Jane</td>++</tr></table>', $result);
    }

    /**
     * @expectedException qfq\UserReportException
     */
    public function testUnknownTokenException() {

        // empty (missing '=')
        $result = $this->report->process('10.sql SELECT "Hello = World"');
    }

    /**
     *
     */
    public function testReportMultipleQuery() {

        // nested 2
        $result = $this->report->process("10.sql = SELECT id FROM Person ORDER BY id LIMIT 2\n10.10.sql = SELECT name FROM Person ORDER BY id LIMIT 2\n");
        $this->assertEquals('1DoeSmith2DoeSmith', $result);

        // nested 3
        $result = $this->report->process("10.sql = SELECT id FROM Person ORDER BY id LIMIT 2\n10.10.sql = SELECT name FROM Person ORDER BY id LIMIT 2\n\n10.10.10.sql = SELECT firstname FROM Person ORDER BY id LIMIT 2\n");
        $this->assertEquals('1DoeJohnJaneSmithJohnJane2DoeJohnJaneSmithJohnJane', $result);

        // nested 2, seq
        $result = $this->report->process("10.sql = SELECT id FROM Person ORDER BY id LIMIT 2\n10.10.sql = SELECT name FROM Person ORDER BY id LIMIT 2\n\n10.20.sql = SELECT firstname FROM Person ORDER BY id LIMIT 2\n");
        $this->assertEquals('1DoeSmithJohnJane2DoeSmithJohnJane', $result);
    }

    /**
     *
     */
    public function testReportLink() {

        // link
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" >http://www.example.com</a>', $result);

        // link, checked
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|C' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" ></a>', $result);

        // linck, checked, text
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|C|t:Hello World' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" > Hello World</a>', $result);

        // link, checked, text, tooltip
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|C|t:Hello World|o:more information' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" title="more information" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="more information" > Hello World</a>', $result);
    }

    /**
     *
     */
    public function testReportLinkRenderMode() {

        // r:0, url + text
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|t:Hello World|r:0' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" >Hello World</a>', $result);

        // r:0, url
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|r:0' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" >http://www.example.com</a>', $result);

        // r:0, text
        $result = $this->report->process("10.sql = SELECT 'r:0|t:Hello World' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('', $result);

        // r:1, url + text
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|t:Hello World|r:1' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" >Hello World</a>', $result);

        // r:1, url
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|r:1' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" >http://www.example.com</a>', $result);

        // r:1, text
        $result = $this->report->process("10.sql = SELECT 'r:1|t:Hello World' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('Hello World', $result);

    }

    /**
     *
     */
    public function testReportMailto() {

        // r:0, mailto
        $result = $this->report->process("10.sql = SELECT 'm:john.doe@example.com' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="mailto:john.doe@example.com" >mailto:john.doe@example.com</a>', $result);

        $result = $this->report->process("10.sql = SELECT 'm:john.doe@example.com|e:0' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="mailto:john.doe@example.com" >mailto:john.doe@example.com</a>', $result);

        //TODO: enable encryption check
//        $result = $this->report->process("10.sql = SELECT 'm:john.doe@example.com|e:1' AS _link FROM Person ORDER BY id LIMIT 1");
//        $this->assertEquals('<a href="mailto:john.doe@example.com" >mailto:john.doe@example.com</a>', $result);

    }

    /**
     *
     */
    public function testReportPageTokenSip() {

        // page with sip (default, without explizit definition)
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" >Person</a>', $result);

        // page with sip (explizit definition via 's')
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|s' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" >Person</a>', $result);

        // page without sip (explizit definition via 's:0')
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|s:0' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="?id=form&r=123&a=hello&type=5&L=3&final=world" >Person</a>', $result);

        // page with sip (explizit definition via 's:1')
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|s:1' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" >Person</a>', $result);

        // page with sip (explizit definition via 's:1')
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" >index.php?id=form&type=5&L=3&s=badcaffee1234</a>', $result);

    }

    /**
     *
     */
    public function testReportPageTokenPicture() {

        // page & picture
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|P:picture.gif' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" ><img alt="picture.gif" src="picture.gif" title="picture.gif" ></a>', $result);

        // page & Edit
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|E' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" class="btn btn-default" title="Edit" ><span class="glyphicon glyphicon-pencil" ></span></a>', $result);

        // page & New
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|N' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" class="btn btn-default" title="New" ><span class="glyphicon glyphicon-plus" ></span></a>', $result);

        // page & Help
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|H' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" class="btn btn-default" title="Help" ><span class="glyphicon glyphicon-question-sign" ></span></a>', $result);

        // page & Information
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|I' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" class="btn btn-default" title="Information" ><span class="glyphicon glyphicon-info-sign" ></span></a>', $result);

        // page & Show
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|S' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-search" ></span></a>', $result);

        // page & Show & Text
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|S|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&type=5&L=3&s=badcaffee1234" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-search" ></span> Person</a>', $result);
    }

    /**
     *
     */
    public function testReportPageTokenBullet() {

        // page & bullet (green)
        $result = $this->report->process("10.sql = SELECT 'p:form|B' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&s=badcaffee1234" ><img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" ></a>', $result);

        // page & bullet (green)
        $result = $this->report->process("10.sql = SELECT 'p:form|B||t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="../../../../../../../../../../scratch/software/typo3/typo3_src-8.7.20/index.php?id=form&s=badcaffee1234" ><img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" > Person</a>', $result);

        $arr = ['blue', 'gray', 'green', 'pink', 'red', 'yellow', 'fake'];
        foreach ($arr as $color) {
            // page & bullet $color
            $result = $this->report->process("10.sql = SELECT 'p:form|B:$color' AS _page FROM Person ORDER BY id LIMIT 1");
            $this->assertEquals(" ></a>", $result);
        }
    }

    /**
     *
     */
    public function testReportPageTokenCheck() {

        // page & bullet (green)
        $result = $this->report->process("10.sql = SELECT 'p:form|C' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" ></a>', $result);

        // page & bullet (green)
        $result = $this->report->process("10.sql = SELECT 'p:form|C|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" > Person</a>', $result);

        $arr = ['blue', 'gray', 'green', 'pink', 'red', 'yellow', 'fake'];
        foreach ($arr as $color) {
            // page & bullet $color
            $result = $this->report->process("10.sql = SELECT 'p:form|C:$color' AS _page FROM Person ORDER BY id LIMIT 1");
            $this->assertEquals("<a href=\"index.php?id=form&s=badcaffee1234\" ><img alt=\"Checked $color\" src=\"typo3conf/ext/qfq/Resources/Public/icons/checked-$color.gif\" title=\"$color\" ></a>", $result);
        }
    }

    /**
     *
     */
    public function testReportPageTokenUrlParam() {

        // page & Url Param
        $result = $this->report->process("10.sql = SELECT 'p:form|U:r=234|s:0|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="?id=form&r=234" >Person</a>', $result);

        // page & Url Param=''
        $result = $this->report->process("10.sql = SELECT 'p:form|U|s:0|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="?id=form" >Person</a>', $result);
    }

    /**
     *
     */
    public function testReportPageTokenToolTip() {

        // page & Tooltip=Tool Tip
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|o:Tool Tip' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" title="Tool Tip" >Person</a>', $result);

        // page & Tooltip=''
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|o' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);
    }

    /**
     *
     */
    public function testReportPageTokenAltText() {

        // page & AltText - no image
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|a' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & AltText - no image
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|a:' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & AltText - no image
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|a:Hello World' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & AltText - image
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|B|a' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" ><img src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" > Person</a>', $result);

        // page & AltText - image
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|B|a:' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" ><img src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" > Person</a>', $result);

        // page & AltText - image
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|B|a:Hello World' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" ><img alt="Hello World" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" > Person</a>', $result);
    }

    /**
     *
     */
    public function testReportPageTokenClass() {

        // page & class (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|c' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & class (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|c:' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & class (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|c:n' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & class (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & class (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & class (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|c:myclass' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" class="myclass" >Person</a>', $result);

    }

    /**
     *
     */
    public function testReportPageTokenTarget() {

        // page & target (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|g' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & target (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|g:' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & target (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|g:_blank' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" target="_blank" >Person</a>', $result);

        // page & target (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|g:_nextframe' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" target="_nextframe" >Person</a>', $result);

    }

    /**
     *
     */
    public function testReportPageTokenQuestion() {
        $js = <<<EOF
id="12345" onClick="var alert = new QfqNS.Alert({ message: 'Please confirm', type: 'info', modal: true, timeout: 0, buttons: [
    { label: 'Ok', eventName: 'ok' },
    { label: 'Cancel',eventName: 'cancel'}
] } );
alert.on('alert.ok', function() {
  window.location = $('#12345').attr('href');
});

alert.show();
return false;"
EOF;

        // _Page:  pagealias, param, Text, Tooltip, Question
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|o:This is a tooltip|q' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question
        $js = str_replace('Please confirm', 'My Question', $js);
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|o:This is a tooltip|q:My Question' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question:Level
        $js = str_replace('info', 'error', $js);
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|o:This is a tooltip|q:My Question:error' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question:Level, Button Ok
        $js = str_replace('Ok', 'YES', $js);
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|o:This is a tooltip|q:My Question:error:YES' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question:Level, Button Cancel
        $js = str_replace('Cancel', 'NO', $js);
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|o:This is a tooltip|q:My Question:error:YES:NO' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question:Level (with escaped colon)
//        $js = str_replace('Cancel', 'NO', $js);
        $js = str_replace('My Question', 'Other Question:some nice value', $js);
        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person|o:This is a tooltip|q:Other Question\\\:some nice value:error:YES:NO' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" ' . $js . ' >Person</a>', $result);
    }

    /**
     *
     */
    public function testReportPageTokenEncryption() {
        // TODO: implement
    }


    /**
     *
     */
    public function testReportPageTokenRight() {

        // page & target (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|R' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person</a>', $result);

        // page & target (empty)
        $result = $this->report->process("10.sql = SELECT 'p:form|t:Person|R|B' AS _page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >Person <img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" ></a>', $result);

    }

    /**
     *
     */
    public function testReportPageFix() {

        $js = <<<EOF
id="12345" onClick="var alert = new QfqNS.Alert({ message: 'Please confirm', type: 'info', modal: true, timeout: 0, buttons: [
    { label: 'Ok', eventName: 'ok' },
    { label: 'Cancel',eventName: 'cancel'}
] } );
alert.on('alert.ok', function() {
  window.location = $('#12345').attr('href');
});

alert.show();
return false;"
EOF;

        // _Page: only pagealias
        $result = $this->report->process("10.sql = SELECT 'form' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" >index.php?id=form&s=badcaffee1234</a>', $result);

        // _Page: pagealias, param
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" >index.php?id=form&type=5&L=3&s=badcaffee1234</a>', $result);

        // _Page: pagealias, param, Text
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" >Person</a>', $result);

        // _Page: pagealias, param, Text, Tooltip
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question
        $js = str_replace('Please confirm', 'My Question', $js);
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question|myclass' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" class="myclass" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question|myclass|mytarget' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" class="myclass" target="mytarget" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 0: URL + Text
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question|myclass|mytarget|0' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" class="myclass" target="mytarget" title="This is a tooltip" ' . $js . ' >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 0: URL
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world||This is a tooltip|My Question|myclass|mytarget|0' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" class="myclass" target="mytarget" title="This is a tooltip" ' . $js . ' >index.php?id=form&type=5&L=3&s=badcaffee1234</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 2: URL
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world||This is a tooltip|My Question|myclass|mytarget|2' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 3: URL  + Text
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question|myclass|mytarget|3' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<span title="This is a tooltip" >Person</span>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 4: URL  + Text
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question|myclass|mytarget|4' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<span title="This is a tooltip" >index.php?id=form&type=5&L=3&s=badcaffee1234</span>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 4: URL
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question|myclass|mytarget|4' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<span title="This is a tooltip" >index.php?id=form&type=5&L=3&s=badcaffee1234</span>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 4: URL + Text
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip|My Question|myclass|mytarget|5' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode 4: URL
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world||This is a tooltip|My Question|myclass|mytarget|5' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode, Sip ON (1)
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip||myclass|mytarget|0|1' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=5&L=3&s=badcaffee1234" class="myclass" target="mytarget" title="This is a tooltip" >Person</a>', $result);

        // _Page:  pagealias, param, Text, Tooltip, Question, Class, Target, Rendermode, Sip OFF (0)
        $result = $this->report->process("10.sql = SELECT 'form&r=123&a=hello&type=5&L=3&final=world|Person|This is a tooltip||myclass|mytarget|0|0' AS _Page FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="?id=form&r=123&a=hello&type=5&L=3&final=world" class="myclass" target="mytarget" title="This is a tooltip" >Person</a>', $result);
    }

    /**
     *
     */
    public function testReportPageC() {

        $js = <<<EOF
id="12345" onClick="var alert = new QfqNS.Alert({ message: 'Please confirm!', type: 'info', modal: true, timeout: 0, buttons: [
    { label: 'Ok', eventName: 'ok' },
    { label: 'Cancel',eventName: 'cancel'}
] } );
alert.on('alert.ok', function() {
  window.location = $('#12345').attr('href');
});

alert.show();
return false;"
EOF;

        // _Page: incl. alert
        $result = $this->report->process("10.sql = SELECT 'p:form' AS _pagec FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" class="btn btn-default" ' . $js . ' >index.php?id=form&s=badcaffee1234</a>', $result);

        // _Page: other than defaults for the alert.
        $js = str_replace('Please confirm!', 'Do you like to open', $js);
        $js = str_replace("type: 'info'", "type: 'success'", $js);
        $js = str_replace('Ok', 'yes', $js);
        $js = str_replace('Cancel', 'no', $js);
        $js = str_replace('timeout: 0', 'timeout: 10000', $js);
        $js = str_replace('modal: true', 'modal: false', $js);
        $result = $this->report->process("10.sql = SELECT 'p:form|q:Do you like to open:success:yes:no:10:0' AS _pagec FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" class="btn btn-default" ' . $js . ' >index.php?id=form&s=badcaffee1234</a>', $result);


        $result = $this->report->process("10.sql = SELECT 'p:form|q:Do you like to open:success:yes:no:10:0|t:click me' AS _pagec FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" class="btn btn-default" ' . $js . ' >click me</a>', $result);

    }

    /**
     *
     */
    public function testReportPageFixC() {

        $js = <<<EOF
id="12345" onClick="var alert = new QfqNS.Alert({ message: 'Please confirm!', type: 'info', modal: true, timeout: 0, buttons: [
    { label: 'Ok', eventName: 'ok' },
    { label: 'Cancel',eventName: 'cancel'}
] } );
alert.on('alert.ok', function() {
  window.location = $('#12345').attr('href');
});

alert.show();
return false;"
EOF;

        // _Page: incl. alert
        $result = $this->report->process("10.sql = SELECT 'form' AS _Pagec FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" class="btn btn-default" ' . $js . ' >index.php?id=form&s=badcaffee1234</a>', $result);

        // _Page: other than defaults for the alert.
        $js = str_replace('Please confirm!', 'Do you like to open', $js);
        $js = str_replace("type: 'info'", "type: 'success'", $js);
        $js = str_replace('Ok', 'yes', $js);
        $js = str_replace('Cancel', 'no', $js);
        $js = str_replace('timeout: 0', 'timeout: 10000', $js);
        $js = str_replace('modal: true', 'modal: false', $js);
        $result = $this->report->process("10.sql = SELECT 'form|||Do you like to open:success:yes:no:10:0' AS _Pagec FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" class="btn btn-default" ' . $js . ' >index.php?id=form&s=badcaffee1234</a>', $result);


        $result = $this->report->process("10.sql = SELECT 'form|click me||Do you like to open:success:yes:no:10:0' AS _Pagec FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&s=badcaffee1234" class="btn btn-default" ' . $js . ' >click me</a>', $result);

    }

    /**
     *
     */
    public function testReportPageD() {

        $js = <<<EOF
id="12345" onClick="var alert = new QfqNS.Alert({ message: 'Do you really want to delete the record?', type: 'warning', modal: true, timeout: 0, buttons: [
    { label: 'Ok', eventName: 'ok' },
    { label: 'Cancel',eventName: 'cancel'}
] } );
alert.on('alert.ok', function() {
  window.location = $('#12345').attr('href');
});

alert.show();
return false;"
EOF;

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'U:table=Person&r=123' AS _paged FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="' . API_DIR . '/' . API_DELETE_PHP . '?s=badcaffee1234" class="btn btn-default" title="Delete" ' . $js . ' ><span class="glyphicon glyphicon-trash" ></span></a>', $result);

        // Check das via '_paged' SIP_MODE_ANSWER and SIP_TARGET_URL has been set.
        $result = Session::get('badcaffee1234');
        $this->assertEquals('__dbIndexData=1&_modeAnswer=html&_targetUrl=localhost&r=123&table=Person', $result);

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'U:form=Person&r=123' AS _paged FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="' . API_DIR . '/' . API_DELETE_PHP . '?s=badcaffee1234" class="btn btn-default" title="Delete" ' . $js . ' ><span class="glyphicon glyphicon-trash" ></span></a>', $result);

        // _paged: other than defaults for the alert.
        $js = str_replace('Do you really want to delete the record?', 'Move to trash?', $js);
        $js = str_replace("type: 'info'", "type: 'success'", $js);
        $js = str_replace('Ok', 'yes', $js);
        $js = str_replace('Cancel', 'no', $js);
        $js = str_replace('timeout: 0', 'timeout: 10000', $js);
        $js = str_replace('modal: true', 'modal: false', $js);
        $js = str_replace("type: 'warning'", "type: 'success'", $js);
        $result = $this->report->process("10.sql = SELECT 'U:table=Person&r=123|q:Move to trash?:success:yes:no:10:0' AS _paged FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="' . API_DIR . '/' . API_DELETE_PHP . '?s=badcaffee1234" class="btn btn-default" title="Delete" ' . $js . ' ><span class="glyphicon glyphicon-trash" ></span></a>', $result);


        $result = $this->report->process("10.sql = SELECT 'U:table=Person&r=123|q:Move to trash?:success:yes:no:10:0|t:click me' AS _paged FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="' . API_DIR . '/' . API_DELETE_PHP . '?s=badcaffee1234" class="btn btn-default" title="Delete" ' . $js . ' ><span class="glyphicon glyphicon-trash" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageFixD() {

        $js = <<<EOF
id="12345" onClick="var alert = new QfqNS.Alert({ message: 'Do you really want to delete the record?', type: 'warning', modal: true, timeout: 0, buttons: [
    { label: 'Ok', eventName: 'ok' },
    { label: 'Cancel',eventName: 'cancel'}
] } );
alert.on('alert.ok', function() {
  window.location = $('#12345').attr('href');
});

alert.show();
return false;"
EOF;

        // _Paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'table=Person&r=123' AS _Paged FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="' . API_DIR . '/' . API_DELETE_PHP . '?s=badcaffee1234" class="btn btn-default" title="Delete" ' . $js . ' ><span class="glyphicon glyphicon-trash" ></span></a>', $result);

        // _Paged: other than defaults for the alert.
        $js = str_replace('Do you really want to delete the record?', 'Move to trash?', $js);
        $js = str_replace("type: 'info'", "type: 'success'", $js);
        $js = str_replace('Ok', 'yes', $js);
        $js = str_replace('Cancel', 'no', $js);
        $js = str_replace('timeout: 0', 'timeout: 10000', $js);
        $js = str_replace('modal: true', 'modal: false', $js);
        $js = str_replace("type: 'warning'", "type: 'success'", $js);
        $result = $this->report->process("10.sql = SELECT 'table=Person&r=123|||Move to trash?:success:yes:no:10:0' AS _Paged FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="' . API_DIR . '/' . API_DELETE_PHP . '?s=badcaffee1234" class="btn btn-default" title="Delete" ' . $js . ' ><span class="glyphicon glyphicon-trash" ></span></a>', $result);


        $result = $this->report->process("10.sql = SELECT 'table=Person&r=123|click me||Move to trash?:success:yes:no:10:0' AS _Paged FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="' . API_DIR . '/' . API_DELETE_PHP . '?s=badcaffee1234" class="btn btn-default" title="Delete" ' . $js . ' ><span class="glyphicon glyphicon-trash" ></span> click me</a>', $result);
    }

    /**
     * Missing r, missing form or table
     * @expectedException qfq\UserReportException
     */
    public function testMissingPagedParameterException1() {
        $this->report->process("10.sql = SELECT '' AS _Paged FROM Person ORDER BY id LIMIT 1");
    }

    /**
     * Missing r, given table
     * @expectedException qfq\UserReportException
     */
    public function testMissingPagedParameterException2() {
        $this->report->process("10.sql = SELECT 'table=Person' AS _Paged FROM Person ORDER BY id LIMIT 1");
    }

    /**
     * Missing r, given form
     * @expectedException qfq\UserReportException
     */
    public function testMissingPagedParameterException3() {
        $this->report->process("10.sql = SELECT 'form=Person' AS _Paged FROM Person ORDER BY id LIMIT 1");
    }

    /**
     * missing form, missing table
     * @expectedException qfq\UserReportException
     */
    public function testMissingPagedParameterException4() {
        $this->report->process("10.sql = SELECT 'r=123' AS _Paged FROM Person ORDER BY id LIMIT 1");
    }

    /**
     * missng r, given table
     * @expectedException qfq\UserReportException
     */
    public function testMissingPagedParameterException5() {
        $this->report->process("10.sql = SELECT 'table=Person&r' AS _Paged FROM Person ORDER BY id LIMIT 1");
    }

    /**
     * missing table, given r
     * @expectedException qfq\UserReportException
     */
    public function testMissingPagedParameterException6() {
        $this->report->process("10.sql = SELECT 'table&r=123' AS _Paged FROM Person ORDER BY id LIMIT 1");
    }

    /**
     * missing form, given r
     * @expectedException qfq\UserReportException
     */
    public function testMissingPagedParameterException7() {
        $this->report->process("10.sql = SELECT 'form&r=123' AS _Paged FROM Person ORDER BY id LIMIT 1");
    }

    /**
     *
     */
    public function testReportPageE() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4' AS _pagee FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Edit" ><span class="glyphicon glyphicon-pencil" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4|t:click me' AS _pagee FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Edit" ><span class="glyphicon glyphicon-pencil" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageFixE() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4' AS _Pagee FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Edit" ><span class="glyphicon glyphicon-pencil" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4|click me' AS _Pagee FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Edit" ><span class="glyphicon glyphicon-pencil" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageH() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4' AS _pageh FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Help" ><span class="glyphicon glyphicon-question-sign" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4|t:click me' AS _pageh FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Help" ><span class="glyphicon glyphicon-question-sign" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageFixH() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4' AS _Pageh FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Help" ><span class="glyphicon glyphicon-question-sign" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4|click me' AS _Pageh FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Help" ><span class="glyphicon glyphicon-question-sign" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageI() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4' AS _pagei FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Information" ><span class="glyphicon glyphicon-info-sign" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4|t:click me' AS _pagei FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Information" ><span class="glyphicon glyphicon-info-sign" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageFixI() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4' AS _Pagei FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Information" ><span class="glyphicon glyphicon-info-sign" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4|click me' AS _Pagei FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Information" ><span class="glyphicon glyphicon-info-sign" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageN() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4' AS _pagen FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="New" ><span class="glyphicon glyphicon-plus" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4|t:click me' AS _pagen FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="New" ><span class="glyphicon glyphicon-plus" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageFixN() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4' AS _Pagen FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="New" ><span class="glyphicon glyphicon-plus" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4|click me' AS _Pagen FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="New" ><span class="glyphicon glyphicon-plus" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageS() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4' AS _pages FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-search" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'p:form&a=1&r=3&type=4|t:click me' AS _pages FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-search" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportPageFixS() {

        // _paged: incl. alert
        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4' AS _Pages FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-search" ></span></a>', $result);

        $result = $this->report->process("10.sql = SELECT 'form&a=1&r=3&type=4|click me' AS _Pages FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="index.php?id=form&type=4&s=badcaffee1234" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-search" ></span> click me</a>', $result);
    }

    /**
     *
     */
    public function testReportBullet() {

        $arr = ['blue', 'gray', 'green', 'pink', 'red', 'yellow', 'fake'];
        foreach ($arr as $color) {
            // bullet $color
            $result = $this->report->process("10.sql = SELECT '$color' AS _bullet FROM Person ORDER BY id LIMIT 1");
            $this->assertEquals("<img alt=\"Bullet $color\" src=\"typo3conf/ext/qfq/Resources/Public/icons/bullet-$color.gif\" title=\"$color\" >", $result);
        }
    }

    /**
     *
     */
    public function testReportCheck() {

        $arr = ['blue', 'gray', 'green', 'pink', 'red', 'yellow', 'fake'];
        foreach ($arr as $color) {
            // check $color
            $result = $this->report->process("10.sql = SELECT '$color' AS _check FROM Person ORDER BY id LIMIT 1");
            $this->assertEquals("<img alt=\"Checked $color\" src=\"typo3conf/ext/qfq/Resources/Public/icons/checked-$color.gif\" title=\"$color\" >", $result);
        }
    }


    /**
     *
     */
    public function testReportPageWrapper() {

        $line = <<<EOF
10.sql = SELECT firstname FROM Person ORDER BY id LIMIT 2
10.head = <table>
10.tail = </table>
10.rbeg = <tr>
10.rend = <br>
10.renr = </tr>
10.fbeg = <td>
10.fend = </td>
10.rsep = --
10.fsep = ++

10.10.sql = SELECT 'nested' FROM (SELECT '') AS fake WHERE '{{10.line.count}}'='1'
10.10.shead = Static head
10.10.stail = Static tail
10.10.head = Dynamic head
10.10.tail = Dynamic tail
10.10.althead = No record found
10.10.altsql = SELECT 'alt sql fired'

EOF;

        $result = $this->report->process($line);
        $expect = "<table><tr><td>John</td><br>Static headDynamic headnestedDynamic tailStatic tail</tr>--<tr><td>Jane</td><br>Static headNo record foundalt sql firedStatic tail</tr></table>";
        $this->assertEquals($expect, $result);
    }

    /**
     *
     */
    public function testReportVariables() {

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals("normal text ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id LIMIT 1\n10.10.sql = SELECT '{{10.hidden}}'");
        $this->assertEquals("normal text hidden", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id LIMIT 1\n10.10.sql = SELECT '{{10.unknown}}'");
        $this->assertEquals("normal text {{10.unknown}}", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id LIMIT 1\n10.10.sql = SELECT '{{fake}}'");
        $this->assertEquals("normal text {{fake}}", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id LIMIT 1\n10.10.sql = SELECT '{{fake:V}}'");
        $this->assertEquals("normal text {{fake:V}}", $result);

        $this->store->setVar('fake', 'hello world ', STORE_VAR);
        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden ' AS _hidden, 'text ' FROM Person ORDER BY id LIMIT 1\n10.10.sql = SELECT '{{fake:V}}'");
        $this->assertEquals("normal text hello world ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id\n10.10.sql = SELECT '{{fake:V}}'");
        $this->assertEquals("normal text hello world normal text hello world ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id\n10.10.sql = SELECT '{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}} '");
        $this->assertEquals("normal text hello world -1-2-0 normal text hello world -2-2-0 ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id\n10.10.sql = SELECT '{{fake:V}}-{{10.line.count}}-{{10.line.total}} '");
        $this->assertEquals("normal text hello world -1-2 normal text hello world -2-2 ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id\n10.10.sql = SELECT '{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.10.line.count}}-{{10.10.line.total}} '");
        $this->assertEquals("normal text hello world -1-2-1-1 normal text hello world -2-2-1-1 ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id\n10.10.sql = SELECT '{{fake:V:::not found}} '");
        $this->assertEquals("normal text hello world  normal text hello world  ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id\n10.10.sql = SELECT '{{fakeDontExist:V:::not found}} '");
        $this->assertEquals("normal text not found normal text not found ", $result);

        $result = $this->report->process("10.sql = SELECT 'normal ', 'hidden' AS _hidden, 'text ' FROM Person ORDER BY id\n10.10.sql = SELECT '{{fakeDontExist:V:::{{EDIT_FORM_PAGE:Y}}}} '");
        $this->assertEquals("normal text form normal text form ", $result);


//        store various
//        store default
        // head.tail,rbeg,.seop mit variabeln
//        5.sql im 10 head abragen
    }

    /**
     *
     */
    public function testReportPageVariables() {

        $line = <<<EOF
10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2
10.head = h:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.tail = t:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.rbeg = rb:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.rend = re:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.renr = rr:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.fbeg = -fb:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.fend = -fe:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.rsep = rs:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.fsep = fs:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
EOF;

        $this->store->setVar('fake', 'hello world', STORE_VAR);
        $result = $this->report->process($line);
        $expect = "h:hello world-1-2-0,rb:hello world-1-2-0,-fb:hello world-1-2-0,Doe-fe:hello world-1-2-0,fs:hello world-1-2-0,-fb:hello world-1-2-0,John-fe:hello world-1-2-0,re:hello world-1-2-0,rr:hello world-1-2-0,rs:hello world-1-2-0,rb:hello world-2-2-0,-fb:hello world-2-2-0,Smith-fe:hello world-2-2-0,fs:hello world-2-2-0,-fb:hello world-2-2-0,Jane-fe:hello world-2-2-0,re:hello world-2-2-0,rr:hello world-2-2-0,t:hello world-2-2-0,";
        $this->assertEquals($expect, $result);

        $line = <<<EOF
10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2
10.10.sql = SELECT ' blue '
10.10.head = h:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.tail = t:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.rbeg = rb:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.rend = re:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.renr = rr:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.fbeg = -fb:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.fend = -fe:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.rsep = rs:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
10.10.fsep = fs:{{fake:V}}-{{10.line.count}}-{{10.line.total}}-{{10.line.insertId}},
EOF;

        $this->store->setVar('fake', 'hello world', STORE_VAR);
        $result = $this->report->process($line);
        $expect = "DoeJohnh:hello world-1-2-0,rb:hello world-1-2-0,-fb:hello world-1-2-0, blue -fe:hello world-1-2-0,re:hello world-1-2-0,rr:hello world-1-2-0,t:hello world-1-2-0,SmithJaneh:hello world-2-2-0,rb:hello world-2-2-0,-fb:hello world-2-2-0, blue -fe:hello world-2-2-0,re:hello world-2-2-0,rr:hello world-2-2-0,t:hello world-2-2-0,";
        $this->assertEquals($expect, $result);

    }


    /**
     *
     */
    public function testReportPageTokenMail() {
        // TODO: implement
    }

    /**
     *
     */
    public function testReportPageTokenRender() {
        // TODO: implement
    }

    /**
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     * @throws UserReportException
     */
    protected function setUp() {

        parent::setUp();

        $GLOBALS["TSFE"] = new FakeTSFEReport();
        $this->eval = new Evaluate($this->store, $this->dbArray[DB_INDEX_DEFAULT]);
        $this->report = new Report(array(), $this->eval, true);

        $this->executeSQLFile(__DIR__ . '/../database/fixtures/Generic.sql', true);
    }
}

/**
 * Class FakeTSFEReport
 */
class FakeTSFEReport {
    public $id = 1;
    public $type = 1;
    public $sys_language_uid = 1;
}
